/* Cyclops
 * FER - SPVP
 * Fakultet elektrotehnike i racunarstva (http://www.fer.hr/)
 * Unska 3, 10000 Zagreb, Hrvatska
 * (c) 2001 FER, Zagreb.
 */


package hr.fer.zesoi.cyclops;

import java.util.*;
import java.awt.*;


/**
 * Base Cyclops simulator class. Preforms simulation.
 * @author Darko Vasi
 * @author Tomislav Petkovi
 * @author Zvonko Kostanjar
 */
public class CyclopsSimulator extends Thread
{

    /** List of all nodes in a network. */
    protected Vector node_list = new Vector();

    /** List of all lines in a network. */
    protected Vector line_list = new Vector();

    /** Source node. */
    protected Node start_node = null;

    /** Source node name. */
    protected String start_node_name = null;

    /** Destination node. */
    protected Node end_node = null;

    /** Destination node name. */
    protected String end_node_name = null;

    /** Flag for packet tracing. */
    protected boolean trace_flag = false;

    /** Flag for stoping simulation. */
    protected boolean running = false;

    /** Flag for destroying thread. */
    protected boolean destroy = false;

    /**
     * Run method.
     */
    public void run()
    {
	while (true)
	    {
		if (running)
		    {
			simulate();
			if (trace_flag)
			    {
				sendPacket();
			    }
			try
			    {
				Thread.sleep(100);
			    }
			catch (InterruptedException exception)
			    {
			    }
		    }
		else
		    {
			try
			    {
				Thread.sleep(100);
			    }
			catch (InterruptedException exception)
			    {
			    }
		    }
		if (destroy)
		    {
			return;
		    }
	    }
    }
    /* run */

    /**
     * Empty constructor.
     */
    public CyclopsSimulator ()
    {
    }
    /* CyclopsSimulator */

    /**
     * Main simulation method. Each simulation cycle is made of
     * two subcycles. First all lines transmits theirs packets and
     * then every node analysises packages and deternimes to which
     * node it shoul be sent.
     */
    public synchronized void simulate ()
    {
	int i;
	Line current_line;
	Node current_node;

	for (i=0; i<line_list.size(); i++)
	    {
		current_line = (Line)line_list.elementAt(i);
		current_line.transmitAll();
	    }
	for (i=0; i<node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_node.transmitAll();
	    }
	for (i=0; i<node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_node.updateRoutingTable();
	    }
    }
    /* simulate */

    /**
     * Topology update.
     */
    public synchronized void topologyUpdate ()
    {
	int i;
	Node current_node;

	for (i=0; i<node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_node.updateRoutingTable();
	    }
    }
    /* topologyUpdate */

    /**
     * Starts/resumes simulation.
     */
    public void startSimulation ()
    {
	running = true;
    }
    /* startSimulation */

    /**
     * Trace packets from source to destination.
     * @param start_name Name of a starting node (source).
     * @param end_name Name of a ending node (destination).
     */
    public void tracePackets (String start_name, String end_name)
    {
	Node start_temp;
	Node end_temp;

	start_temp = findNode(start_name);
	if (start_temp == null)
	    {
		return;
	    }
	end_temp = findNode(end_name);
	if (end_temp == null)
	    {
		return;
	    }
	start_node = start_temp;
	start_node_name = start_name;
	end_node = end_temp;
	end_node_name = end_name;
	trace_flag = true;
    }
    /* tracePackets */

    /**
     * Sends packet from source to destination. Source and destination
     * are start_node and end_node.
     */
    public void sendPacket()
    {
	Node temp;

	temp = findNode(start_node_name);
	if (temp != null)
	    {
		temp = findNode(end_node_name);
		if (temp != null)
		    {
			Packet traced = new Packet(120, start_node, end_node, 14);
			traced.setDrawingFlag(true);
			start_node.receive(traced);
		    }
	    }
    }
    /* sendPacket */

    /**
     * Adds dummy packets. Packets with negative ttl
     * will live forever.
     * @param number Number of packets.
     * @param time_to_live TTL.
     */
    public void addDummyPackets(int number, int time_to_live)
    {
	int i;
	int j;
	Node current_node;
	Packet packet;
	boolean running_flag;

	running_flag = running;
	running = false;
	/* Add dummy packets. */
	for (i=0, j=0; i<number; i++)
	    {
		packet = new Packet();
		current_node = (Node)node_list.elementAt(j);
		j++;
		if (j >= node_list.size())
		    {
			j = 0;
		    }
		packet.setTimeToLive(time_to_live);
		current_node.receive(packet);
	    }
	running = running_flag;
    }
    /* addDummyPackets */

    /**
     * Stops/suspends simulation.
     */
    public void stopSimulation ()
    {
	running = false;
    }
    /* stopSimulation */

    /**
     * Ends simulation.
     */
    public void endSimulation ()
    {
	destroy = true;
    }
    /* endSimulation */

    /**
     * Adds node to network.
     * @param new_node Node.
     */
    public void addNode (Node new_node)
    {
	node_list.addElement(new_node);
    }
    /* addNode */

    /**
     * Finds node with given name.
     * @param node_name Node name.
     * @return Returns node reference.
     */
    public Node findNode (String node_name)
    {
	int i;
	String current_name;
	Node current_node;

	for (i=0; i!=node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_name = current_node.getName();
		if (current_name.equals(node_name))
		    {
			return current_node;
		    }
	    }
	return null;
    }
    /* findNode*/

    /**
     * Deletes node with given name.
     * @param node_name Node name.
     * @return Returns node reference.
     */
    public Node deleteNode (String node_name)
    {
	int i;
	String current_name;
	Node current_node = null;
	Line current_line;

	for (i=0; i!=node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_name = current_node.getName();
		if (current_name.equals(node_name))
		    {
			current_node.delete();
			node_list.removeElementAt(i);
			break;
		    }
	    }
	for (i=0; i!=line_list.size(); i++)
	    {
		current_line = (Line)line_list.elementAt(i);
		if (current_line.isDisconnected())
		    {
			line_list.removeElementAt(i);
			/* Next element becomes current element, so i-- is required. */
			i--;
		    }
	    }
	return current_node;
    }
    /* deleteNode*/

    /**
     * Gets number of nodes.
     * @return Number of nodes.
     */
    public int numberOfNodes ()
    {
	return node_list.size();
    }
    /* numberOfNodes */

    /**
     * Returns node at specified index. Index must be less
     * then number returnd by getNumberOfNodes().
     * @param index Node index.
     * @return Node reference.
     */
    public Node nodeAt(int index)
    {
	return (Node)node_list.elementAt(index);
    }
    /* nodeAt */

    /**
     * Returns line at specified index. Index must be less
     * then number returnd by getNumberOfLines().
     * @param index Line index.
     * @return Line reference.
     */
    public Line lineAt(int index)
    {
	return (Line)line_list.elementAt(index);
    }
    /* lineAt */

    /**
     * Gets number of lines.
     * @return Number of lines.
     */
    public int numberOfLines ()
    {
	return line_list.size();
    }
    /* numberOfLines */

    /**
     * Adds line to network.
     * @param new_line Line.
     */
    public void addLine (Line new_line)
    {
	line_list.addElement(new_line);
    }
    /* addLine */

    /**
     * Deletes line from network.
     * @param first_node First node name.
     * @param second_node Second node name.
     * @return Returns line reference.
     */
    public Line deleteLine (String first_node, String second_node)
    {
	Node first;
	Node second;
	Line current_line;
	int i;

	first = findNode(first_node);
	second = findNode(second_node);
	if (null == first) return null;
	if (null == second) return null;
	current_line = null;
	for (i=0; i!=line_list.size(); i++)
	    {
		current_line = (Line)line_list.elementAt(i);
		if (current_line.areEndingNodes(first, second))
		    {
			current_line.delete();
			line_list.removeElementAt(i);
			break;
		    }
	    }
	return current_line;
    }
    /* deleteLine */

    /**
     * Paints network on screen.
     * @param g Graphics reference.
     * @param metric Font metrics for displaying information.
     * @param node_color Node color.
     * @param line_color Line color.
     * @param left_packet_color Color for packets traveling form left to right.
     * @param right_packet_color Color for packets traveling form right to left.
     * @param traced_packet_color Color for traced packets.
     * @param show_line_capacity Capacity flag.
     * @param show_all_packets Show all packets flag.
     */
    public synchronized void paint (Graphics g, FontMetrics metrics,
				    Color node_color, Color line_color,
				    Color left_packet_color, Color right_packet_color,
				    Color traced_packet_color,
				    boolean show_line_capacity, boolean show_all_packets
				    )
    {
	int i;
	Line current_line;
	Node current_node;

	for (i=0; i!=line_list.size(); i++)
	    {
		current_line = (Line)line_list.elementAt(i);
		current_line.paint(g, metrics,
				   line_color, left_packet_color, right_packet_color,
				   traced_packet_color,
				   show_line_capacity, show_all_packets);
	    }
	for (i=0; i!=node_list.size(); i++)
	    {
		current_node = (Node)node_list.elementAt(i);
		current_node.paint(g, metrics, node_color);
	    }

    }
    /* paint */

}
/* CyclopsSimulator */
