/* Cyclops
 * FER - SPVP
 * Fakultet elektrotehnike i racunarstva (http://www.fer.hr/)
 * Unska 3, 10000 Zagreb, Hrvatska
 * (c) 2001 FER, Zagreb.
 */

package hr.fer.zesoi.cyclops;

import java.util.*;
import java.awt.*;


/**
 * Node class. The Node class implements a generic network node..
 * @author Tomislav Petkovi 
 */
public class Node
{

    /** RIP table. */
    public RIPTable routing_table;

    /** Node name. Every node should have one. */
    protected String name;
    
    /** x screen coordinate. */
    protected int x_position = 20;

    /** y screen coordinate. */
    protected int y_position = 20;

    /** Queue for all incoming packages. */
    protected NodeQueue receive_buffer = new NodeQueue();

    /** List of all conecting lines. */
    protected Vector connecting_lines = new Vector();

    /** List of all connecting nodes. */
    protected Vector connecting_nodes = new Vector();



    /**
     * Empty constructor. Creates completly isolated node.
     */
    public Node ()
    {
	name = "Izolirani cvor";
	x_position = (int)(50 + 200 * Math.random());
	y_position = (int)(50 + 200 * Math.random());
	this.routing_table = new RIPTable(this);
    }
    /* Node */

    /**
     * Normal constructor. Creates named node. All nodes should
     * have differen names.
     * @param name Node name. 
     */
    public Node (String name)
    {
	x_position = (int)(50 + 200 * Math.random());
	y_position = (int)(50 + 200 * Math.random());
	this.name = name;
	this.routing_table = new RIPTable(this);
    }
    /* Node */

    /**
     * Receives one packet and stores it into a buffer.
     * @param packet Packet object.
     */
    public void receive (Object packet)
    {
	receive_buffer.put(packet);
    }
    /* receive */

    /**
     * Updates routing table.
     */
    public void updateRoutingTable ()
    {
	int i;
	Node other_node;

	for (i=0; i<connecting_nodes.size(); i++)
	    {
		other_node = (Node)connecting_nodes.elementAt(i);
		routing_table.updateTable(other_node);
	    }
	routing_table.garbageCollect();
    }
    /* updateRoutingTable */

    /**
     * Transmits all packages currently stored in output buffer.
     */
    public void transmitAll ()
    {
	int i;
	Packet packet;
	Node destination;
	Node next_node;
	LineEnd line;

	while ( !receive_buffer.isEmpty() )
	    {
		packet = (Packet)receive_buffer.get();
		destination = packet.getDestinationNode();
		if (destination == this)
		    {
			/* Discard packet. */
		    }
		else if (packet.hop() != 0)
		    {
			next_node = routing_table.findNextNodeForRouteTo(destination);
			if (next_node == null)
			    {
				i = (int)(Math.random()*connecting_lines.size());
				if (!connecting_lines.isEmpty())
				    {
					line = (LineEnd)connecting_lines.elementAt(i);
					line.receive(packet);
				    }
			    }
			else
			    {
				i = connecting_nodes.indexOf(next_node);
				if (i >= 0)
				    {
					line = (LineEnd)connecting_lines.elementAt(i);
					line.receive(packet);
				    }
			    }
			/* else */
		    }
		/* else if */
	    }
	/* while */
    }
    /* transmitAll */

    /**
     * Attach line end.
     * @param line_end Line end object.
     * @param other_node Node at the other line end.
     */
    public void attach (LineEnd line_end, Node other_node)
    {
	connecting_lines.addElement(line_end);
	connecting_nodes.addElement(other_node);
	routing_table.addElement(other_node, other_node, 1);
    }
    /* attach */

    /**
     * Detach line end.
     * @param line_end Line end object.
     */
    public void detach (LineEnd line_end)
    {
	int index;
	Node node;

	index = connecting_lines.indexOf(line_end);
	if (0 <= index)
	    {
		connecting_lines.removeElementAt(index);
		node = (Node)connecting_nodes.elementAt(index);
		connecting_nodes.removeElementAt(index);
		routing_table.setCostToInfinity(node);
	    }
    }
    /* detach */

    /**
     * Deletes node.
     */
    public void delete ()
    {
	int index;

	while (!connecting_lines.isEmpty())
	    {
		LineEnd line = (LineEnd)connecting_lines.lastElement();
		line.delete();
	    }
    }
    /* delete */

    /**
     * Detach node.
     * @param other_node Node object.
     */
    public void detach (Node other_node)
    {
	int index;

	index = connecting_nodes.indexOf(other_node);
	if (0 <= index)
	    {
		connecting_lines.removeElementAt(index);
		connecting_nodes.removeElementAt(index);
		routing_table.setCostToInfinity(other_node);
	    }
    }
    /* detach */

    /**
     * Gets RIP table.
     * @return Returns RIP table reference.
     */
    public RIPTable getRIPTable ()
    {
	return routing_table;
    }
    /* getRIPTable */

    /**
     * Sets screen coordinates.
     * @param x x screen coordinate.
     * @param y y screen coordinate.
     */
    public void setCoordinates (int x, int y)
    {
	x_position = x;
	y_position = y;
    }
    /* setCoordinates */

    /**
     * Gets node name.
     * @return Returns node name.
     */
    public String getName ()
    {
	return this.name;
    }
    /* nodeName */

    /**
     * Gets x cooridnate.
     * @return x screen coordinate.
     */
    public int getXCoordinate ()
    {
	return x_position;
    }
    /* getXCoordinate */

    /**
     * Gets y cooridnate.
     * @return y screen coordinate.
     */
    public int getYCoordinate ()
    {
	return y_position;
    }
    /* getYCoordinate */

    /**
     * Sets x cooridnate.
     * @param x x screen coordinate.
     */
    public void setXCoordinate (int x)
    {
	x_position = x;
    }
    /* setXCoordinate */

    /**
     * Sets y cooridnate.
     * @param y y screen coordinate.
     */
    public void getYCoordinate (int y)
    {
	y_position = y;
    }
    /* setYCoordinate */

    /**
     * Draws node on screen.
     * @param g Graphics reference.
     * @param metrics Font metrics required for node name.
     * @param fill_color Color.
     */
    public void paint (Graphics g, FontMetrics metrics, Color fill_color)
    {
	int w = metrics.stringWidth(this.name) + 10;
	int h = metrics.getHeight() + 4;
	g.setColor(fill_color);
	g.fillRect(x_position - w/2, y_position - h/2, w, h);
	g.setColor(Color.black);
	g.drawRect(x_position - w/2, y_position - h/2, w - 1, h - 1);
	g.drawString(this.name,
		     x_position - (w - 10) / 2,
                     y_position - (h - 4) / 2 + metrics.getAscent()
		     );
    }
    /* paint */

}
/* Node */


/**
 * NodeQueue class implements simple packet storage buffer.
 */
class NodeQueue
{

    /** Packet storage. */
    private Stack storage_buffer = new Stack();

    /**
     * Default constructor.
     */
    public NodeQueue ()
    {
    }
    /* NodeQueue */

    /**
     * Test for empty buffer. Returns true if empty.
     * @return Buffer status.
     */
    public boolean isEmpty ()
    {
	return storage_buffer.isEmpty();
    }
    /* isEmpty */

    /**
     * Puts one packet in the buffer.
     * @param packet New element.
     */
    public void put (Object packet)
    {
	storage_buffer.push(packet);
    }
    /* put */

    /**
     * Gets next element (packet) and removes it from the
     * buffer. Generates exception if buffer is empty.
     * @return Returns buffer element (packet).
     */
    public Object get ()
    {
	Object packet;

	packet = storage_buffer.pop();
	return packet;
    }
    /* get */

    /**
     * Peeks at the first element.
     * @return Returns first element.
     */
    public Object peek ()
    {
	return storage_buffer.peek();
    }
    /* peek */

}
/* NodeQueue */
