/* Cyclops
 * FER - SPVP
 * Fakultet elektrotehnike i racunarstva (http://www.fer.hr/)
 * Unska 3, 10000 Zagreb, Hrvatska
 * (c) 2001 FER, Zagreb.
 */

package hr.fer.zesoi.cyclops;

import java.awt.*;


/**
 * The Packet class implements a generic packet. Generic packet
 * has a variable size storage for any data. Methods that are
 * implemented are required by Line class to be able to destinguish
 * packet direction.
 * @author Darko Vasi
 * @author Tomislav Petkovi
 * @author Zvonko Kostanjar
 */
public class Packet
{

    /** A varible size buffer for packet's contents. */
    protected StringBuffer contents;

    /** Packet length. */
    protected int packet_length = 500;

    /** Source node. Set by first Node. */
    protected Node source_node = null;

    /** Destination node. Set by first Node. */
    protected Node destination_node = null;

    /** Time to live. */
    protected int time_to_live = 40;

    /** Drawing flag. */
    protected boolean draw = false;

    /**
     * Default empty constructor. Constructs a 500 bytes package.
     */
    public Packet ()
    {
	contents = new StringBuffer (packet_length);
    }
    /* Packet */

    /**
     * Constructructs a packet with inital size specified by length argument.
     * @param length Packet length.
     */
    public Packet (int length)
    {
	if(length < 0) length = 500;
	packet_length = length;
	contents = new StringBuffer(packet_length);
    }
    /* Packet */

    /**
     * Constructructs a packet with inital size specified by length argument
     * and sets source and destination node.
     * @param length Packet length.
     * @param source Source node.
     * @param destination Destination node.
     */
    public Packet (int length, Node source, Node destination)
    {
	if(length < 0) length = 500;
	packet_length = length;
	source_node = source;
	destination_node = destination;
	contents = new StringBuffer(packet_length);
    }
    /* Packet */

    /**
     * Constructructs a packet with inital ttl specified by ttl argument
     * and sets source and destination node.
     * @param length Packet length.
     * @param source Source node.
     * @param destination Destination node.
     * @param ttl Time to live.
     */
    public Packet (int length, Node source, Node destination, int ttl)
    {
	if(length < 0) length = 500;
	packet_length = length;
	source_node = source;
	destination_node = destination;
	contents = new StringBuffer(packet_length);
	time_to_live = ttl;
    }
    /* Packet */

    /**
     * Gets packet contents.
     * @return Packet contents.
     */
    public StringBuffer getContents ()
    {
	return contents;
    }
    /* getContents */

    /**
     * Sets packet contents.
     * @param contents New packet contetns.
     */
    public void setContents (StringBuffer contents)
    {
	this.contents.setLength(0);
	this.contents.append(contents);
	this.contents.setLength(packet_length);
    }
    /* setContents */

    /**
     * Gets packet length.
     * @return Packet length.
     */
    public int getLength ()
    {
	return packet_length;
    }
    /* getLength */

    /**
     * Sets TTL counter. If the given time is
     * negative, packet will live forever (or until simulation
     * is terminated).
     * @param time_to_live TTL.
     */
    public void setTimeToLive (int time_to_live)
    {
	this.time_to_live = time_to_live;
    }
    /* setTimeToLive */

    /**
     * Decreases TTL counter. When zero is hit, packet will be discarded.
     * @return Returns TTL.
     */
    public int hop ()
    {
	if (time_to_live > 0)
	    {
		time_to_live -= 1;
	    }
	return time_to_live;
    }
    /* hop */

    /**
     * Sets source node.
     * @param sourceNode Source node.
     */
    public void setSourceNode (Node source_node)
    {
	this.source_node = source_node;
    }
    /* setSourceNode */

    /**
     * Sets destination node.
     * @param destinationNode Destination node.
     */
    public void setDestinationNode (Node destination_node)
    {
	this.destination_node = destination_node;
    }
    /* setDestinationNode */

    /**
     * Gets source node.
     * @return Source node.
     */
    public Node getSourceNode ()
    {
	return source_node;
    }
    /* getSourceNode */

    /**
     * Gets destination node.
     * @return Destination node.
     */
    public Node getDestinationNode ()
    {
	return destination_node;
    }
    /* getDestinationNode */

    /**
     * Gets status of drawing flag.
     * Drawing flag is used to distinguish important packets (packets
     * that have the drawing flag set will always be drawn).
     * @return Drawing flag.
     */
    public boolean getDrawingFlag ()
    {
	return draw;
    }
    /* getDrawingFlag */

    /**
     * Sets drawing flag.
     * @param flag Drawing flag.
     */
    public void setDrawingFlag (boolean flag)
    {
	draw = flag;
    }
    /* setDrawingFlag */

    /**
     * Draws packet at location (x, y).
     * @param g Graphics reference.
     * @param metrics Font metrics.
     * @param fill_color Packet color.
     * @param x x coordinate.
     * @param y y coordinate
     */
    public void paint (Graphics g, FontMetrics metrics, Color fill_color, int x, int y)
    {
	g.setColor(fill_color);
	g.fillOval(x, y, 8, 8);
	g.setColor(Color.black);
	g.drawOval(x, y, 8, 8);
    }
    /* paint */

}
/* Packet */
